@@ -152,6 +152,46 @@ apollo_fmnl <- function(fmnl_settings, functionality){
       return(P)
     }
     
+    fmnl_settings$fmnl_diagnostics <- function(inputs, apollo_inputs, data=TRUE, param=TRUE){
+      
+    
+        # Table describing dependent variable
+        choicematrix <- matrix(0, nrow=4, ncol=inputs$nAlt, 
+                               dimnames=list(c("Times available",
+                                               "Observations with non-zero share",
+                                               "Average share overall",
+                                               "Average share when available"),
+                                             inputs$altnames))
+        for(a in 1:inputs$nAlt){
+          choicematrix[1,a] <- ifelse(length(inputs$avail[[a]])==1 && inputs$avail[[a]]==1, 
+                                      inputs$nObs, sum(inputs$avail[[a]]) )
+          choicematrix[2,a] <- sum(inputs$choiceShares[[a]]>0)
+          choicematrix[3,a] <- ifelse(choicematrix[1,a]>0, mean(inputs$choiceShares[[a]]), 0)
+          choicematrix[4,a] <- ifelse(choicematrix[1,a]>0, sum(inputs$choiceShares[[a]])/choicematrix[1,a], 0)
+        }
+        # Print table
+        if(!apollo_inputs$silent & data){
+          apollo_print("\n")
+          apollo_print(paste0('Overview of choices for ', toupper(inputs$modeltype), ' model component ', 
+                              ifelse(inputs$componentName=='model', '', inputs$componentName), ':'))
+          print(round(choicematrix,2))
+          
+          # Print warnings
+          for(a in 1:inputs$nAlt){
+            if(choicematrix[2,a]==0) apollo_print(paste0('WARNING: Alternative "', inputs$altnames[a], '" is never chosen in model component "', inputs$componentName, '".'))
+            if(choicematrix[4,a]==1) apollo_print(paste0('WARNING: Alternative "', inputs$altnames[a], '" is always given the full choice share when available in model component "', inputs$componentName, '".'))
+          }
+          #if(inputs$avail_set==TRUE & !apollo_inputs$silent) apollo_print(paste0('Availability not provided (or some elements are NA) for model component ', inputs$componentName,'. Full availability assumed.'))
+        }
+      
+        return(invisible(TRUE))
+
+    }
+    
+    
+    # Store model type
+    fmnl_settings$modelType <- modelType
+    
     # Construct necessary input for gradient (including gradient of utilities)
     apollo_beta <- tryCatch(get("apollo_beta", envir=parent.frame(), inherits=TRUE),
                             error=function(e) return(NULL))
@@ -199,7 +239,7 @@ apollo_fmnl <- function(fmnl_settings, functionality){
     if(!apollo_inputs$apollo_control$noValidation) apollo_validate(fmnl_settings, modelType, 
                                                                    functionality, apollo_inputs)
     
-    if(!apollo_inputs$apollo_control$noDiagnostics) apollo_diagnostics(fmnl_settings, modelType, apollo_inputs)
+    if(!apollo_inputs$apollo_control$noDiagnostics) fmnl_settings$fmnl_diagnostics(fmnl_settings, apollo_inputs)
     
     testL = fmnl_settings$probs_fmnl(fmnl_settings, all=FALSE)
     if(any(!fmnl_settings$rows)) testL <- apollo_insertRows(testL, fmnl_settings$rows, 1)
@@ -323,8 +363,8 @@ apollo_fmnl <- function(fmnl_settings, functionality){
   if(functionality=='report'){
     P <- list()
     apollo_inputs$silent <- FALSE
-    P$data  <- capture.output(apollo_diagnostics(fmnl_settings, modelType, apollo_inputs, param=FALSE))
-    P$param <- capture.output(apollo_diagnostics(fmnl_settings, modelType, apollo_inputs, data =FALSE))
+    P$data  <- capture.output(fmnl_settings$fmnl_diagnostics(fmnl_settings, apollo_inputs, param=FALSE))
+    P$param <- capture.output(fmnl_settings$fmnl_diagnostics(fmnl_settings, apollo_inputs, data =FALSE))
     return(P)
   }
   
