@@ -202,6 +202,45 @@ apollo_mdcev <- function(mdcev_settings,functionality){
       return(P)
     }
     
+    mdcev_settings$mdcev_diagnostics <- function(inputs, apollo_inputs, data=TRUE, param=TRUE){
+      
+      
+        # Table describing dependent variable
+        choicematrix <- matrix(0, nrow=4, ncol=inputs$nAlt, 
+                               dimnames=list(c("Times available","Observations in which chosen",
+                                               "Average consumption when available",
+                                               "Average consumption when chosen"),
+                                             inputs$altnames))
+        for(a in 1:inputs$nAlt){
+          choicematrix[1,a] <- ifelse(length(inputs$avail[[a]])==1 && inputs$avail[[a]]==1, 
+                                      inputs$nObs, sum(inputs$avail[[a]]) )
+          choicematrix[2,a] <- sum(inputs$discrete_choice[[a]])
+          choicematrix[3,a] <- ifelse(choicematrix[1,a]>0, sum(inputs$continuousChoice[[a]])/choicematrix[1,a], 0)
+          choicematrix[4,a] <- ifelse(choicematrix[2,a]>0, sum(inputs$continuousChoice[[a]])/choicematrix[2,a], 0)
+        }
+        # Print table
+        if(!apollo_inputs$silent & data){
+          apollo_print("\n")
+          apollo_print(paste0('Overview of choices for ', toupper(inputs$modeltype), ' model component ', 
+                              ifelse(inputs$componentName=='model', '', inputs$componentName), ':'))
+          print(round(choicematrix,2))
+          
+          # Print warnings
+          for(a in 1:inputs$nAlt){
+            if(choicematrix[2,a]==0) apollo_print(paste0('WARNING: Alternative "', inputs$altnames[a], '" is never chosen in model component "', inputs$componentName, '".'))
+            if(choicematrix[2,a]==choicematrix[1,a] && inputs$altnames[a]!=inputs$outside) apollo_print(paste0('WARNING: Alternative "', inputs$altnames[a], '" is always chosen when available in model component "', inputs$componentName, '".'))
+          }
+          #if(inputs$avail_set==TRUE & !apollo_inputs$silent) apollo_print(paste0('Availability not provided (or some elements are NA) for model component ', inputs$componentName,'. Full availability assumed.'))
+        }
+        
+      #### return ####
+      return(invisible(TRUE))
+    }
+    
+    
+    # Store model type
+    mdcev_settings$modelType <- modelType
+    
     # Construct necessary input for gradient (including gradient of utilities)
     apollo_beta <- tryCatch(get("apollo_beta", envir=parent.frame(), inherits=TRUE),
                             error=function(e) return(NULL))
@@ -282,7 +321,7 @@ apollo_mdcev <- function(mdcev_settings,functionality){
     if(test) apollo_validate(mdcev_settings, modelType, functionality, apollo_inputs)
     
     test <- !apollo_inputs$apollo_control$noDiagnostics
-    if(test) apollo_diagnostics(mdcev_settings, modelType, apollo_inputs)
+    if(test) mdcev_settings$mdcev_diagnostics(mdcev_settings, apollo_inputs)
     
     testL = mdcev_settings$probs_MDCEV(mdcev_settings)
     if(any(!mdcev_settings$rows)) testL <- apollo_insertRows(testL, mdcev_settings$rows, 1)
@@ -511,8 +550,8 @@ apollo_mdcev <- function(mdcev_settings,functionality){
   if(functionality=='report'){
     P <- list()
     apollo_inputs$silent <- FALSE
-    P$data  <- capture.output(apollo_diagnostics(mdcev_settings, modelType, apollo_inputs, param=FALSE))
-    P$param <- capture.output(apollo_diagnostics(mdcev_settings, modelType, apollo_inputs, data =FALSE))
+    P$data  <- capture.output(mdcev_settings$mdcev_diagnostics(mdcev_settings, apollo_inputs, param=FALSE))
+    P$param <- capture.output(mdcev_settings$mdcev_diagnostics(mdcev_settings, apollo_inputs, data =FALSE))
     return(P)
   }
   
