@@ -47,17 +47,21 @@ apollo_panelProd <- function(P, apollo_inputs, functionality){
 
   apollo_control = apollo_inputs[["apollo_control"]]
   if(apollo_control$HB==TRUE) stop('Function apollo_panelProd should not be used when apollo_control$HB==TRUE!')
-  if(!apollo_control$panelData) stop('Panel data setting not used, so multiplying over choices using apollo_panelProd not applicable!')
-    
+  
   inputIsList <- is.list(P)
   indivID <- apollo_inputs$database[, apollo_control$indivID]
+  
+  
 
   # ############################################### #
   #### functionalities with untransformed return ####
   # ############################################### #
   
   if(functionality%in%c("components","prediction","preprocess","raw", "report")) return(P)
-
+  
+  # Additional check
+  if(!apollo_control$panelData) stop('Panel data setting not used, so multiplying over choices using apollo_panelProd not applicable!')
+  
   # ########################################## #
   #### functionality="gradient"             ####
   # ########################################## #
@@ -110,11 +114,14 @@ apollo_panelProd <- function(P, apollo_inputs, functionality){
     for(j in 1:length(P)){
       if(is.array(P[[j]]) && length(dim(P[[j]]))==3) stop('Need to average over intra-individual draws first before multiplying over choices!')
       if(is.vector(P[[j]]) || (is.matrix(P[[j]]) && !apollo_control$workInLogs) ){
+        if(is.vector(P[[j]]) && length(P[[j]])==length(unique(indivID))) stop('You attempted to call the function apollo_panelProd at a stage where the probabilities are already at the individual level!')
+        if(is.matrix(P[[j]]) && nrow(P[[j]])==length(unique(indivID))) stop('You attempted to call the function apollo_panelProd at a stage where the probabilities are already at the individual level!')
         P[[j]] <- rowsum(log(P[[j]]), group=indivID, reorder=FALSE)
         if(!apollo_control$workInLogs) P[[j]] <- exp(P[[j]])
       }
       if(apollo_control$panelData && is.matrix(P[[j]]) && apollo_control$workInLogs && nrow(P[[j]])==length(indivID)){
         # approach to use if working in logs with mixing
+        if(is.matrix(P[[j]]) && nrow(P[[j]])==length(unique(indivID))) stop('You attempted to call the function apollo_panelProd at a stage where the probabilities are already at the individual level!')
         B    <- rowsum(log(P[[j]]), group=indivID, reorder=FALSE) # nIndiv x nDraws
         Bbar <- apply(B, MARGIN=1, function(r) mean(r[is.finite(r)]) ) # nIndiv x 1 ### FIX: 15/5/2020
         P[[j]] <- ifelse(is.finite(Bbar), Bbar + log( rowMeans(exp(B-Bbar)) ), -Inf) # nIndiv x 1 ### FIX 18/5/2020
