@@ -14,10 +14,10 @@
   txt <- paste0("Apollo ", apolloVersion,
   "\nhttp://www.ApolloChoiceModelling.com",
   "\nSee url for a detailed manual, examples and a user forum.",
-  "\nSign up to to user forum to receive updates on new releases.")
+  "\nSign up to the user forum to receive updates on new releases.")
   
   ### Warning if more than six months old
-  releaseDate <- as.POSIXct('2022-01-18', format='%Y-%m-%d')
+  releaseDate <- as.POSIXct('2022-07-01', format='%Y-%m-%d')
   isOld <- as.POSIXct(Sys.Date()) > (releaseDate + 6*30.4*24*60*60)
   if(isOld) txt <- paste0(txt, 
                           '\n\nYour version of Apollo is more than six months old.',
